package main

import (
	"log"
	"time"

	"git.joco.dk/sng/fermentord/internal/controllers"
	"git.joco.dk/sng/fermentord/pkg/temperature"
	"github.com/getsentry/sentry-go"
	"github.com/spf13/viper"
)

func loadConfiguration() *controllers.ControllerConfig {
	hub := sentry.CurrentHub().Clone()
	defer hub.Flush(10 * time.Second)

	viper.SetDefault("http.port", 8000)
	viper.SetDefault("main.data_path", "./data.txt")
	viper.SetDefault("nats.stream", "FERMENTOR")
	viper.SetDefault("nats.subject.state", "FERMENTOR.state")
	viper.SetDefault("nats.subject.temp", "FERMENTOR.temp")
	viper.SetDefault("nats.url", "nats.service.consul")
	viper.SetDefault("pid.kd", 2.0)
	viper.SetDefault("pid.ki", 0.0001)
	viper.SetDefault("pid.kp", 2.0)
	viper.SetDefault("limits.heater_grace_time_secs", 1800)
	viper.SetDefault("limits.max_chamber_temp", 40)
	viper.SetDefault("limits.max_cooler_runtime_secs", 86400)
	viper.SetDefault("limits.min_chamber_temp", 5)
	viper.SetDefault("limits.min_cooler_cooldown_secs", 300)
	viper.SetDefault("limits.min_cooler_runtime_secs", 300)
	viper.SetDefault("sensors.weight", 0.8)

	viper.AddConfigPath("/etc")
	viper.AddConfigPath("/usr/local/etc")
	viper.AddConfigPath(".")
	viper.SetConfigName("fermentord")
	viper.SetConfigType("toml")

	if err := viper.ReadInConfig(); err != nil {
		log.Printf("Error loading configuration: %v", err)
	}

	config := &controllers.ControllerConfig{}
	if err := viper.Unmarshal(config); err != nil {
		hub.CaptureException(err)
		log.Fatal(err)
	}

	return config
}

func reloadConfiguration(config *controllers.ControllerConfig, ctrl *controllers.ChamberController) {
	log.Printf("Reloading configuration")

	temperature.ConfigUpdates <- []string{
		config.Sensors.Ambient,
		config.Sensors.Chamber,
		config.Sensors.Wort,
	}

	ctrl.ConfigUpdates <- *config
}
